/**************************************************************************************************/
/*                                      INCLUDE_FILES                                             */
/**************************************************************************************************/
/* 标准C库头文件 */
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include "gpio.h"
#include "common_sys.h"
/**************************************************************************************************/
/*                                      DEFINES                                                   */
/**************************************************************************************************/
/*GPIO defination*/
int g_gpio_export_fd = -1;
int g_gpio_unexport_fd = -1;

#define GPIO_READY() (g_gpio_export_fd>=0 && g_gpio_unexport_fd>=0)
#define GPIO_OPEN_FILE(fd,dir,flag,fail_ret) \
	do{\
	fd = open(dir,flag);\
	if (-1 == fd)\
	{\
		DBG_VERBOSE("open %s fail !\n",dir);\
		return fail_ret;\
	}\
	}while (0)
#define GPIO_WRITE_FILE(fd,str,size,fail_ret) \
	do{\
	if (-1 == write(fd,str,size))\
	{\
		GPIO_CLOSE_FILE(fd);\
		DBG_VERBOSE("write gpio fail !\n");\
		return fail_ret;\
	}\
	}while (0)
#define GPIO_READ_FILE(fd,p_value,size,fail_ret) \
	do{\
	if (-1 == read(fd,p_value,size))\
	{\
		GPIO_CLOSE_FILE(fd);\
		DBG_VERBOSE("read gpio fail !\n");\
		return fail_ret;\
	}\
	}while (0)
#define GPIO_CLOSE_FILE(fd) do{close(fd);}while (0)


/**************************************************************************************************/
/*                                      FUNCTIONS                                                 */
/**************************************************************************************************/

/*!
 *\fn		ad_misc_gpio_init
 *\brief	GPIO initialization
 *
 *\param[in]	unit	SOC unit(default 0 for marvell)
 *\param[in]	gpio	gpio pin number
 *\param[out]	N/A
 *
 *\return
 *\retval	ERR_NO_ERROR/ERR_HW_OP_FAIL
 *
 *\note	 this function must be done before any other GPIO config
 */
int ad_misc_gpio_init(usp_uint8_t unit, usp_uint8_t gpio)
{
	char gpio_str[8] = {0};
	
	GPIO_OPEN_FILE(g_gpio_export_fd,"/sys/class/gpio/export",O_WRONLY,ERR_HW_OP_FAIL);
	sprintf(gpio_str,"%d",gpio);
	GPIO_WRITE_FILE(g_gpio_export_fd,gpio_str,sizeof(gpio_str),ERR_HW_OP_FAIL);
	GPIO_CLOSE_FILE(g_gpio_export_fd);
	return ERR_NO_ERROR;
}

/*!
 *\fn		ad_misc_gpio_de_init
 *\brief	GPIO de-init proc
 *
 *\param[in]	unit	SOC unit(default 0 for marvell)
 *\param[in]	gpio	gpio pin number
 *\param[out]	N/A
 *
 *\return
 *\retval	ERR_NO_ERROR/ERR_HW_OP_FAIL
 *
 *\note	 this function must be done before any other GPIO config
 */

int ad_misc_gpio_de_init(usp_uint8_t unit, usp_uint8_t gpio)
{
	char gpio_str[8] = {0};

	GPIO_OPEN_FILE(g_gpio_unexport_fd,"/sys/class/gpio/unexport",O_WRONLY,ERR_HW_OP_FAIL);
	sprintf(gpio_str,"%d",gpio);
	GPIO_WRITE_FILE(g_gpio_unexport_fd,gpio_str,sizeof(gpio_str),ERR_HW_OP_FAIL);
	GPIO_CLOSE_FILE(g_gpio_unexport_fd);
	return ERR_NO_ERROR;
}

/*!
 *\fn		ad_misc_gpio_dir_set
 *\brief	GPIO direction config
 *
 *\param[in]	unit	SOC unit(default 0 for marvell)
 *\param[in]	gpio	gpio pin number
 *\param[in]	dir		gpio pin direction
  *\param[in]	val		init value if direction is out
 *\param[out]	N/A
 *
 *\return
 *\retval	ERR_NO_ERROR/ERR_HW_OP_FAIL
 *
 *\note
 */
int ad_misc_gpio_dir_set(usp_uint8_t unit, usp_uint8_t gpio, usp_uint8_t dir, usp_uint val)
{
	char direction_file[GPIO_FNAME_LEN] = {0};
	int direction_fd = 0;
	char *direction_str_val[]={"low","high"};
	
	sprintf(direction_file,"/sys/class/gpio/gpio%d/direction",gpio);
	GPIO_OPEN_FILE(direction_fd,direction_file,O_RDWR,ERR_HW_OP_FAIL);
	if (GPIO_DIRECTION_INPUT == dir)
	{
		GPIO_WRITE_FILE(direction_fd,"in",strlen("in"),ERR_HW_OP_FAIL);
	}
	else if (GPIO_DIRECTION_OUTPUT == dir)
	{
		GPIO_WRITE_FILE(direction_fd,direction_str_val[val],strlen(direction_str_val[val]),ERR_HW_OP_FAIL);
	}
	else
	{
		return ERR_BAD_PARAM;
	}
	GPIO_CLOSE_FILE(direction_fd);
	return ERR_NO_ERROR;
}

/*!
 *\fn		ad_misc_gpio_set
 *\brief	GPIO value config
 *
 *\param[in]	unit	SOC unit(default 0 for marvell)
 *\param[in]	flag	pin type in marvell: PIN_TYPE_XXX
 *\param[in]	gpio	gpio pin number
 *\param[in]	val		gpio pin value

 *\param[out]	N/A
 *
 *\return
 *\retval	ERR_NO_ERROR/ERR_HW_OP_FAIL
 *
 *\note
 */
int ad_misc_gpio_set(usp_uint8_t unit, usp_uint8_t flag, usp_uint8_t gpio, usp_uint val)
{
	char value_file[GPIO_FNAME_LEN] = {0};
	int value_fd = 0;
	char *val_str[]={"0","1"};

	if (flag == PIN_TYPE_GPIO)
	{
		sprintf(value_file,"/sys/class/gpio/gpio%d/value",gpio);
		GPIO_OPEN_FILE(value_fd,value_file,O_RDWR,ERR_HW_OP_FAIL);
		GPIO_WRITE_FILE(value_fd,val_str[val],strlen(val_str[val]),ERR_HW_OP_FAIL);
		GPIO_CLOSE_FILE(value_fd);
	}
	/*Serial pin can only be used as GPO*/
	else if (flag == PIN_TYPE_SERIAL)
	{
		
	}
	else
	{
		//do nothing
	}
	return ERR_NO_ERROR;
}