/*
 * $Id: sync.c 1.34 Broadcom SDK $
 * $Copyright: Copyright 2012 Broadcom Corporation.
 * This program is the proprietary software of Broadcom Corporation
 * and/or its licensors, and may only be used, duplicated, modified
 * or distributed pursuant to the terms and conditions of a separate,
 * written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized
 * License, Broadcom grants no license (express or implied), right
 * to use, or waiver of any kind with respect to the Software, and
 * Broadcom expressly reserves all rights in and to the Software
 * and all intellectual property rights therein.  IF YOU HAVE
 * NO AUTHORIZED LICENSE, THEN YOU HAVE NO RIGHT TO USE THIS SOFTWARE
 * IN ANY WAY, AND SHOULD IMMEDIATELY NOTIFY BROADCOM AND DISCONTINUE
 * ALL USE OF THE SOFTWARE.
 *
 * Except as expressly set forth in the Authorized License,
 *
 * 1.     This program, including its structure, sequence and organization,
 * constitutes the valuable trade secrets of Broadcom, and you shall use
 * all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of
 * Broadcom integrated circuit products.
 *
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS
 * PROVIDED "AS IS" AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES,
 * REPRESENTATIONS OR WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY,
 * OR OTHERWISE, WITH RESPECT TO THE SOFTWARE.  BROADCOM SPECIFICALLY
 * DISCLAIMS ANY AND ALL IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
 * NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES,
 * ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION OR
 * CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING
 * OUT OF USE OR PERFORMANCE OF THE SOFTWARE.
 *
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL
 * BROADCOM OR ITS LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL,
 * INCIDENTAL, SPECIAL, INDIRECT, OR EXEMPLARY DAMAGES WHATSOEVER
 * ARISING OUT OF OR IN ANY WAY RELATING TO YOUR USE OF OR INABILITY
 * TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF
 * THE AMOUNT ACTUALLY PAID FOR THE SOFTWARE ITSELF OR USD 1.00,
 * WHICHEVER IS GREATER. THESE LIMITATIONS SHALL APPLY NOTWITHSTANDING
 * ANY FAILURE OF ESSENTIAL PURPOSE OF ANY LIMITED REMEDY.$
 *
 * File: 	sync.c
 * Purpose:	Defines pal routines for mutexes and semaphores
 *
 * Mutex and Binary Semaphore abstraction
 *
 * Note: the PAL mutex abstraction is required to allow the same mutex
 * to be taken recursively by the same thread without deadlock.
 *
 * The POSIX mutex used here has been further abstracted to ensure this.
 */

/* Macro: _XOPEN_SOURCE
If you define this macro, functionality described in the X/Open Portability Guide is included. This is a superset of the POSIX.1 and POSIX.2 functionality and in fact _POSIX_SOURCE and _POSIX_C_SOURCE are automatically defined.
*/
/* #define _XOPEN_SOURCE 600 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <signal.h>
#include <pthread.h>
#include <semaphore.h>
#include <time.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#ifndef NDEBUG
#define NDEBUG
#endif
#include <assert.h>
#include "usp_type.h"
#include <usp_pal_sync.h>
#include <usp_pal_thread.h>

#if defined (__STRICT_ANSI__)
#define NO_CONTROL_C
#endif

#ifdef TP_PAL_DEBUG
static unsigned int _pal_sem_count_curr;
static unsigned int _pal_sem_count_max;
static unsigned int _pal_mutex_count_curr;
static unsigned int _pal_mutex_count_max;
static unsigned int _pal_cond_count_curr;
static unsigned int _pal_cond_count_max;
#define PAL_SEM_RESOURCE_USAGE_INCR(a_curr, a_max, ilock)               \
        (a_curr)++;                                                       \
        (a_max) = ((a_curr) > (a_max)) ? (a_curr) : (a_max)

#define PAL_SEM_RESOURCE_USAGE_DECR(a_curr) (a_curr)-- /* parasoft-suppress PREPROC-04 "Internal use for resource statistic" */
#endif

/*
 * Function:
 *      pal_sem_resource_usage_get
 * Purpose:
 *      Provides count of active sem and maximum sem allocation
 * Parameters:
 *      sem_curr - Current semaphore allocation.
 *      sem_max - Maximum semaphore allocation.
 */

void
pal_sem_resource_usage_get(unsigned int *sem_curr, unsigned int *sem_max)
{
#ifdef TP_PAL_DEBUG
    if (sem_curr != NULL) {
        *sem_curr = _pal_sem_count_curr;
    }
    if (sem_max != NULL) {
        *sem_max = _pal_sem_count_max;
    }
#endif
}

/*
 * Function:
 *      pal_mutex_resource_usage_get
 * Purpose:
 *      Provides count of active mutex and maximum mutex allocation
 * Parameters:
 *      mutex_curr - Current mutex allocation.
 *      mutex_max - Maximum mutex allocation.
 */

void
pal_mutex_resource_usage_get(unsigned int *mutex_curr, unsigned int *mutex_max)
{
#ifdef TP_PAL_DEBUG
    if (mutex_curr != NULL) {
        *mutex_curr = _pal_mutex_count_curr;
    }
    if (mutex_max != NULL) {
        *mutex_max = _pal_mutex_count_max;
    }
#endif
}

/*
 * Keyboard interrupt protection
 *
 *   When a thread is running on a console, the user could Control-C
 *   while a mutex is held by the thread.  Control-C results in a signal
 *   that longjmp's somewhere else.  We prevent this from happening by
 *   blocking Control-C signals while any mutex is held.
 */

#ifndef NO_CONTROL_C
static int ctrl_c_depth = 0;
int ctrl_c_blocked = 0;
static sigset_t ctrl_c_sigset;
static int ctrl_c_initialized;
#endif

static void
ctrl_c_block(void)
{
#ifndef NO_CONTROL_C
    if (pal_thread_self() == pal_thread_main_get()) {
	if (ctrl_c_depth++ == 0) {
	    sigprocmask(SIG_BLOCK, &ctrl_c_sigset, NULL);
        ctrl_c_blocked = 1;
	}
    }
#endif
}

static void
ctrl_c_unblock(void)
{
#ifndef NO_CONTROL_C
    if (pal_thread_self() == pal_thread_main_get()) {
	assert(ctrl_c_depth > 0); /* parasoft-suppress JSF-187 "Make sense in debug mode only" */
    --ctrl_c_depth;
	if (ctrl_c_depth == 0) {
	    sigprocmask(SIG_UNBLOCK, &ctrl_c_sigset, NULL);
        ctrl_c_blocked = 0;
	}
    }
#endif
}

/*
 * recursive_mutex_t
 *
 *   This is an abstract type built on the POSIX mutex that allows a
 *   mutex to be taken recursively by the same thread without deadlock.
 *
 *   The Linux version of pthreads supports recursive mutexes
 *   (a non-portable extension to posix). In this case, we
 *   use the Linux support instead of our own.
 */

typedef struct recursive_mutex_s {
    pthread_mutex_t	mutex;
    char		*desc;
    pal_thread_t	owner;
    int			recurse_count;

#ifdef TP_PAL_DEBUG
#define FILE_LOC_NAME_MAX 128
    unsigned int ctrl_c_blk;
    unsigned int take_count;
    unsigned int give_count;
    unsigned int tk_exc_gv_ind;
    char         prev_file_tk_location[FILE_LOC_NAME_MAX];
    char         last_file_tk_location[FILE_LOC_NAME_MAX];

    char         prev_file_gv_location[FILE_LOC_NAME_MAX];
    char         last_file_gv_location[FILE_LOC_NAME_MAX];
#endif

} recursive_mutex_t;


int pal_mutex_take_intern(pal_mutex_t m, int usec);
int pal_mutex_give_intern(pal_mutex_t m);

#ifdef TP_PAL_DEBUG
#include "string.h"

#define MUTEX_DBG_ARR_MAX 5000
static recursive_mutex_t *mutex_dbg_arr_ptr[MUTEX_DBG_ARR_MAX] = {0};

/*If you want to set GDB on a breakpoint for certain events */
void pal_mutext_dbg_break(void)
{
    printf("\n_got Debugger Break Function Indication. Examine Call Stack Now\n");
}

int pal_mutex_take_tp_debug(pal_mutex_t m, int usec, const char *take_loc, int line)
{
    char line_num[8] = {'\0'};
    int ret_val;
    recursive_mutex_t	*rm = (recursive_mutex_t *) m;

    /* print error info */
    if (rm == NULL) 
    {
        printf("\r\n[%s],%d, try to take null mutex. assert failed.\r\n", take_loc, line);
    }
    assert(rm); /* parasoft-suppress JSF-187 "assert" */

    ret_val = pal_mutex_take_intern(m, usec);

    rm->take_count++;

    sprintf(line_num, ":%d", line);
	strncpy(rm->prev_file_tk_location, rm->last_file_tk_location, FILE_LOC_NAME_MAX - 6);
	strncpy(rm->last_file_tk_location, take_loc, FILE_LOC_NAME_MAX - 6);
    strcat(rm->last_file_tk_location, line_num);

#if 0
   /* Detect recursion usage and flag*/
    if (rm->take_count > rm->give_count + 1 && !rm->tk_exc_gv_ind) {
        rm->tk_exc_gv_ind = 1; /* Only print first occurence */
        printf("\r\n_occuur recursion take, and this will only be printed once\r\n");
        printf ("\n_mTX %s TK:%d EXCEEDS GV:%d\n", rm->desc, rm->take_count, rm->give_count);
        printf ("AT: Last Loc:%s  Prev Loc: %s",
                rm->last_file_tk_location, rm->prev_file_tk_location);
        pal_mutext_dbg_break();
    }
#endif

    return ret_val;
}


int pal_mutex_give_tp_debug(pal_mutex_t m, const char *give_loc, int line)
{
    char line_num[8] = {'\0'};
    recursive_mutex_t	*rm = (recursive_mutex_t *) m;
    assert(rm); /* parasoft-suppress JSF-187 "assert" */
    rm->give_count++;

    sprintf(line_num, ":%d", line);
	strncpy(rm->prev_file_gv_location, rm->last_file_gv_location, FILE_LOC_NAME_MAX - 6);
	strncpy(rm->last_file_gv_location, give_loc, FILE_LOC_NAME_MAX - 6);
    strcat(rm->last_file_gv_location, line_num);

    if (rm->give_count > rm->take_count) {
        printf ("\n_eRROR: MTX %s GV:%d EXCEEDS TK:%d\n", rm->desc, rm->give_count, rm->take_count);
        pal_mutext_dbg_break();
    }

    return pal_mutex_give_intern(m);
}
#else
int
pal_mutex_take_tp_debug(pal_mutex_t m, int usec, const char *give_loc, int line)
{
    return pal_mutex_take_intern(m, usec);
}

int
pal_mutex_give_tp_debug(pal_mutex_t m, const char *give_loc, int line)
{
    return pal_mutex_give_intern(m);
}
#endif

void pal_mutex_dbg_dump(void)
{
#ifdef TP_PAL_DEBUG
    int i = 0;
    char temp_ch;

    for (i = 0; i < MUTEX_DBG_ARR_MAX; i++) {
        /* Find nonempty slots */
        if (mutex_dbg_arr_ptr[i] != 0) {
            recursive_mutex_t *rm = mutex_dbg_arr_ptr[i];


            /* If MUTEX was created but never taken or given don't display it */
            if (rm->take_count || rm->give_count) {
                printf ("\n\n_mUTEX STATS[%d]  For:%s  Owner:ox%x CTRL_C_Depth:%d\n",
                        i, rm->desc, (unsigned int)rm->owner, rm->ctrl_c_blk);

                printf("\n_t_CNT:%d G_CNT:%d \n_pREV_T_LOC:%s  LST_T_LOC:%s\n_pREV_G_LOC:%s  LST_G_LOC:%s",
                       rm->take_count, rm->give_count,
                       rm->prev_file_tk_location, rm->last_file_tk_location,
                       rm->prev_file_gv_location, rm->last_file_gv_location);
            }

            if (rm->ctrl_c_blk > 0) {
                printf ("\n_wARNING: CTRL_C Left DISABLED??");
                temp_ch = getchar(); /*Pause  */
            }

            if (rm->take_count != rm->give_count) {
                printf ("\n_wARNING: Take != Give");
                temp_ch = getchar(); /*Pause  */
            }
        }
    }
#endif
}

static pal_mutex_t
_pal_mutex_create(char *desc)
{
    recursive_mutex_t	*rm = NULL;
    pthread_mutexattr_t attr;

#ifdef TP_PAL_DEBUG
    int i = 0;
#endif

#ifndef NO_CONTROL_C
    if (!ctrl_c_initialized) {
        sigemptyset(&ctrl_c_sigset);
        sigaddset(&ctrl_c_sigset, SIGINT);
        ctrl_c_initialized = 1;
    }
#endif

    rm = malloc(sizeof (recursive_mutex_t));
    if (rm == NULL) {
	return NULL;
    }

    rm->desc = desc;
    pthread_mutexattr_init(&attr);
    rm->owner = 0;
    rm->recurse_count = 0;

#ifdef TP_PAL_DEBUG
    rm->ctrl_c_blk = 0;
    rm->take_count = 0;
    rm->give_count = 0;
    rm->tk_exc_gv_ind = 0;

    for (i = 0; i < FILE_LOC_NAME_MAX; i++) {
        rm->prev_file_tk_location[i] = '\0';
        rm->last_file_tk_location[i] = '\0';

        rm->prev_file_gv_location[i] = '\0';
        rm->last_file_gv_location[i] = '\0';
    }

    for (i = 0; i < MUTEX_DBG_ARR_MAX; i++) {
        /* Find an empty slot */
        if (mutex_dbg_arr_ptr[i] == 0) {
            mutex_dbg_arr_ptr[i] = rm;
            break;
        }
    }

#endif

    pthread_mutex_init(&rm->mutex, &attr);

    pthread_mutexattr_destroy(&attr);   /* release attr */
#ifdef TP_PAL_DEBUG
        PAL_SEM_RESOURCE_USAGE_INCR(
            _pal_mutex_count_curr,
            _pal_mutex_count_max,
            ilock);
#endif
    return (pal_mutex_t) rm;
}

/*
 * Mutex and semaphore abstraction
 */

pal_mutex_t
pal_mutex_create(char *desc)
{
    return _pal_mutex_create(desc);

}

void
pal_mutex_destroy(pal_mutex_t m)
{
    recursive_mutex_t	*rm = (recursive_mutex_t *) m;

#ifdef TP_PAL_DEBUG
    int i;
#endif

    assert(rm); /* parasoft-suppress JSF-187 "assert" */

    /* Check for pending mutex unlocks */
    if ((rm->recurse_count > 0) || (rm->owner != 0)) {
        char thread_name[PAL_THREAD_NAME_MAX_LEN];
        pal_thread_name(rm->owner, thread_name, sizeof (thread_name));
        printf("WARNING: Mutex \"%s\" has not been unlocked before being destroyed.\n",
                        rm->desc);
        printf("\t Current owner is \"%s\"(%p) \n", thread_name, (void*)rm->owner);
    }
#ifdef TP_PAL_DEBUG
    assert(rm->recurse_count == 0); /* parasoft-suppress JSF-187 "assert" */
    assert(rm->owner == 0);         /* parasoft-suppress JSF-187 "assert" */
#endif /* TP_PAL_DEBUG */

#ifdef TP_PAL_DEBUG
    assert(rm->take_count == rm->give_count);   /* parasoft-suppress JSF-187 "assert" */

    for (i = 0; i < FILE_LOC_NAME_MAX; i++) {
        rm->prev_file_tk_location[i] = '\0';
        rm->last_file_tk_location[i] = '\0';

        rm->prev_file_gv_location[i] = '\0';
        rm->last_file_gv_location[i] = '\0';
    }

    for (i = 0; i < MUTEX_DBG_ARR_MAX; i++) {
        /* Find it's slot and mark empty*/
        if (mutex_dbg_arr_ptr[i] == rm) {
            mutex_dbg_arr_ptr[i] = 0;
            break;
        }
    }

#endif

    pthread_mutex_destroy(&rm->mutex);

    free(rm);

#ifdef TP_PAL_DEBUG
        PAL_SEM_RESOURCE_USAGE_DECR(
            _pal_mutex_count_curr);
#endif
}

int
pal_compute_timeout(struct timespec *ts, int usec)
{
    int sec;
    unsigned int nsecs;

#ifdef CLOCK_REALTIME
    if (clock_gettime(CLOCK_REALTIME, ts) == 0) {
        ;
    }
    else
#endif
    {
        struct timeval	ltv;

        /* Fall back to RTC if realtime clock unavailable */
        gettimeofday(&ltv, 0);
        ts->tv_sec = ltv.tv_sec;
        ts->tv_nsec = ltv.tv_usec * 1000;
    }
    /* Add in the delay */
    ts->tv_sec += usec / 1000000ULL;

    /* compute new nsecs */
    nsecs = ts->tv_nsec + (usec % 1000000ULL) * 1000;

    /* detect and handle rollover */
    if (nsecs < ts->tv_nsec) {
        ts->tv_sec += 1;
        nsecs -= (1000000ULL * 1000);
    }
    ts->tv_nsec = nsecs;

    /* Normalize if needed */
    sec = ts->tv_nsec / (1000000ULL * 1000);
    if (sec) {
        ts->tv_sec += sec;
        ts->tv_nsec = ts->tv_nsec % (1000000ULL * 1000);
    }

    /* indicate that we successfully got the time */
    return 1;
}

int
pal_mutex_take_intern(pal_mutex_t m, int usec)
{
    recursive_mutex_t	*rm = (recursive_mutex_t *) m;
    int			err = 0;

    pal_thread_t	myself = pal_thread_self();

    assert(rm); /* parasoft-suppress JSF-187 "assert" */
    if (rm->owner == myself) {

        rm->recurse_count++;

        return 0;
    }

    ctrl_c_block();

#ifdef TP_PAL_DEBUG
    rm->ctrl_c_blk++;
#endif

    if (usec == pal_mutex_FOREVER) {
        do {
            err = pthread_mutex_lock(&rm->mutex);
        } while (err != 0 && errno == EINTR);
    }
    else {
        int		time_wait = 1;

        /* Retry algorithm with exponential backoff */

        while (1) {
            err = pthread_mutex_trylock(&rm->mutex);

            if (err != EBUSY) {
                break;		/* Done (0), or error other than EBUSY */
            }

            if (time_wait > usec) {
                time_wait = usec;
            }

            pal_usleep(time_wait);

            usec -= time_wait;

            if (usec == 0) {
                err = ETIMEDOUT;
                break;
            }

            time_wait *= 2;
            if (time_wait > 100000) {
                time_wait = 100000;
            }
        }
    }

    if (err != 0) {
        ctrl_c_unblock();

#ifdef TP_PAL_DEBUG
        printf("\n\n_eRROR in TAKING MUTEX \n\n");
        rm->ctrl_c_blk--;
#endif

        return -1;
    }
    assert(rm->owner == 0); /* parasoft-suppress JSF-187 "assert" */
    rm->owner = myself;

    return 0;
} /* parasoft-suppress BD-TRS-LOCK "The resource will be release in the give function" */



int
pal_mutex_give_intern(pal_mutex_t m)
{
    recursive_mutex_t	*rm = (recursive_mutex_t *) m;
    int			err;

    assert(rm);     /* parasoft-suppress JSF-187 "assert" */

    if ((rm->owner != pal_thread_self())) {
        assert(rm->owner == pal_thread_self()); /* parasoft-suppress JSF-187 "assert" */
    }

    if (rm->recurse_count > 0) {
        rm->recurse_count--;
        return 0;
    }

    rm->owner = 0;

    err = pthread_mutex_unlock(&rm->mutex);
    ctrl_c_unblock();

#ifdef TP_PAL_DEBUG
    rm->ctrl_c_blk--;
#endif


    return err ? -1 : 0;
}


/*
 * Wrapper class to hold additional info
 * along with the semaphore.
 */
typedef struct {
    sem_t       s;
    char        *desc;
} wrapped_sem_t;

pal_sem_t
pal_sem_create(char *desc, int binary, int initial_count)
{
    wrapped_sem_t *s = NULL;

    /* Ignore binary for now */
    s = malloc(sizeof (wrapped_sem_t));
    if (s == NULL) {
        return NULL;
    }
    sem_init(&s->s, 0, initial_count);
    s->desc = desc;

#ifdef TP_PAL_DEBUG
        PAL_SEM_RESOURCE_USAGE_INCR(
            _pal_sem_count_curr,
            _pal_sem_count_max,
            ilock);
#endif

    return (pal_sem_t) s;
}

void
pal_sem_destroy(pal_sem_t b)
{
    wrapped_sem_t *s = (wrapped_sem_t *) b;

    assert(s);  /* parasoft-suppress JSF-187 "assert" */

    sem_destroy(&s->s);

    free(s);

#ifdef TP_PAL_DEBUG
    PAL_SEM_RESOURCE_USAGE_DECR(
        _pal_sem_count_curr);
#endif
}

int
pal_sem_take(pal_sem_t b, int usec)
{
    wrapped_sem_t		*s = (wrapped_sem_t *) b;
    int			err = 0;
    struct timespec abs_timeout;

    if (usec == pal_sem_FOREVER) {
        do {
            err = sem_wait(&s->s);
        } while (err != 0 && errno == EINTR);
    }
    else if (usec == pal_sem_NOWAIT)
    {
        err = sem_trywait(&s->s);
    }
    else
    {
        pal_compute_timeout(&abs_timeout, usec);
        err = sem_timedwait(&s->s, &abs_timeout);
    }

    return err ? -1 : 0;
}

int
pal_sem_give(pal_sem_t b)
{
    wrapped_sem_t		*s = (wrapped_sem_t *) b;
    int			err;

    err = sem_post(&s->s);

    return err ? -1 : 0;
}

typedef struct {
    sem_t       *s;
    char        *desc;
} wrapped_proc_sem_t;
/* processes syncing */
/* 有名信号量, desc为有名信号量文件路径 */
pal_sem_t
pal_proc_sem_create(char *desc, int binary, int initial_count)
{
    wrapped_proc_sem_t *s = NULL;

    assert(NULL != desc);   /* parasoft-suppress JSF-187 "assert" */

    /* Ignore binary for now */
    s = malloc(sizeof (wrapped_sem_t));
    if (s == NULL) {
        return NULL;
    }
    s->s = sem_open(desc, O_CREAT, 00644, initial_count); /* parasoft-suppress JSF-149 "Using octal number in openning file" */
    s->desc = desc;

#ifdef TP_PAL_DEBUG
        PAL_SEM_RESOURCE_USAGE_INCR(
            _pal_sem_count_curr,
            _pal_sem_count_max,
            ilock);
#endif

    return (pal_sem_t) s;
}

pal_sem_t
pal_proc_sem_open(char *desc)
{
    wrapped_proc_sem_t *s = NULL;

    assert(NULL != desc);   /* parasoft-suppress JSF-187 "assert" */

    /* Ignore binary for now */
    s = malloc(sizeof (wrapped_sem_t));
    if (s == NULL) {
        return NULL;
    }
    s->s = sem_open(desc, 0, 00644, 0); /* parasoft-suppress JSF-149 "Using octal number in openning file" */
    s->desc = desc;

#ifdef TP_PAL_DEBUG
        PAL_SEM_RESOURCE_USAGE_INCR(
            _pal_sem_count_curr,
            _pal_sem_count_max,
            ilock);
#endif

    return (pal_sem_t) s;
}

void
pal_proc_sem_destroy(pal_sem_t b)
{
    wrapped_sem_t *s = (wrapped_sem_t *) b;

    assert(&s);  /* parasoft-suppress JSF-187 "assert" */

    sem_close(&s->s);
    sem_unlink(s->desc);

    free(s);

#ifdef TP_PAL_DEBUG
    PAL_SEM_RESOURCE_USAGE_DECR(
        _pal_sem_count_curr);
#endif
}

void
pal_proc_sem_close(pal_sem_t b)
{
    wrapped_sem_t *s = (wrapped_sem_t *) b;

    assert(&s);  /* parasoft-suppress JSF-187 "assert" */

    sem_close(&s->s);

    free(s);

#ifdef TP_PAL_DEBUG
    PAL_SEM_RESOURCE_USAGE_DECR(
        _pal_sem_count_curr);
#endif
}

int
pal_proc_sem_take(pal_sem_t b)
{
    wrapped_sem_t		*s = (wrapped_sem_t *) b;
    int			err = 0;

    err = sem_wait(&s->s);

    return err ? -1 : 0;
}

int
pal_proc_sem_give(pal_sem_t b)
{
    wrapped_sem_t		*s = (wrapped_sem_t *) b;
    int			err;

    err = sem_post(&s->s);

    return err ? -1 : 0;
}

/*
 * wrapped_cond_t
 *
 *   Wrapper class to hold additional info
 *  along with the condition.
 */

typedef struct wrapped_cond_s {
    pthread_cond_t	cond;
    char		    *desc;
    pal_thread_t	owner;
} wrapped_cond_t;


pal_cond_t
pal_cond_create(char *desc)
{
    wrapped_cond_t *c = NULL;

    c = malloc(sizeof (wrapped_cond_t));
    if (c == NULL) {
        return NULL;
    }

    pthread_cond_init(&c->cond, NULL);

    c->desc = desc;

#ifdef TP_PAL_DEBUG
        PAL_SEM_RESOURCE_USAGE_INCR(
            _pal_cond_count_curr,
            _pal_cond_count_max,
            ilock);
#endif

    return (pal_cond_t) c;
}

void
pal_cond_destroy(pal_cond_t b)
{
    wrapped_cond_t *c = (wrapped_cond_t *) b;

    assert(c);  /* parasoft-suppress JSF-187 "assert" */

    pthread_cond_destroy(&c->cond);

    free(c);

#ifdef TP_PAL_DEBUG
    PAL_SEM_RESOURCE_USAGE_DECR(
        _pal_cond_count_curr);
#endif
}

int
pal_cond_wait(pal_cond_t b, pal_mutex_t m, int usec)
{
    struct timespec timeout = {0};
    struct timeval now = {0};
    wrapped_cond_t	*c = (wrapped_cond_t *) b;
    recursive_mutex_t *mu= (recursive_mutex_t*)m;
    int	err = 0;
    pal_thread_t myself = pal_thread_self();

    if (mu->recurse_count > 0)
    {
        mu->recurse_count--;
        return 0;
    }

    mu->owner = 0;

#ifdef TP_PAL_DEBUG
    mu->give_count++;
#endif

    if (usec == pal_cond_FOREVER)
    {
        err = pthread_cond_wait(&c->cond, &mu->mutex);
    }
    else
    {
        gettimeofday(&now, NULL);
        timeout.tv_sec = now.tv_sec + usec / 1000;
        timeout.tv_nsec = now.tv_usec + usec % 1000;
        err = pthread_cond_timedwait(&c->cond, &mu->mutex, &timeout);
    }

    if (mu->owner == myself)
    {
        mu->recurse_count++;
    }
    mu->owner = myself;

#ifdef TP_PAL_DEBUG
    mu->take_count++;
#endif

    if (ETIMEDOUT == err)
    {
        return PAL_COND_TIMEDOUT;
    }
    else
    {
        return 0;
    }
}

int
pal_cond_signal(pal_cond_t b)
{
    wrapped_cond_t	*c = (wrapped_cond_t *) b;
    int	err;

    err = pthread_cond_signal(&c->cond);

    return err;
}


