/*
 * Atheros AP147 reference board support
 *
 * Copyright (c) 2013 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <linux/gpio.h>
#include <linux/spinlock.h>
#include <linux/platform_device.h>
#include <linux/ath9k_platform.h>
#include <linux/ar8216_platform.h>

#include <asm/mach-ath79/ar71xx_regs.h>
#include <asm/delay.h>

#include "common.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-usb.h"
#include "dev-wmac.h"
#include "machtypes.h"
#include "pci.h"

#define AP147_GPIO_LED_WPS		13
#define AP147_GPIO_LED_STATUS		13

#if 0
#define AP147_GPIO_LED_WLAN		12
#define AP147_GPIO_LED_WAN		4
#define AP147_GPIO_LED_LAN1		16
#define AP147_GPIO_LED_LAN2		15
#define AP147_GPIO_LED_LAN3		14
#define AP147_GPIO_LED_LAN4		11
#else
#define AP147_GPIO_LED_WLAN		14
#define AP147_GPIO_LED_WAN	    16	
#define AP147_GPIO_LED_LAN1		1
#define AP147_GPIO_LED_LAN2		2
#define AP147_GPIO_LED_LAN3		3
#define AP147_GPIO_LED_LAN4		4
#endif /* 0 */

#define AP147_GPIO_BTN_WPS		17

#define AP147_KEYS_POLL_INTERVAL	20	/* msecs */
#define AP147_KEYS_DEBOUNCE_INTERVAL	(3 * AP147_KEYS_POLL_INTERVAL)

#define AP147_MAC0_OFFSET		0
#define AP147_MAC1_OFFSET		6
#define AP147_WMAC_CALDATA_OFFSET	0x1000

#define TL_WVR300V4_GPIO_BTN_RESET	    13
#define TL_WVR300V4_GPIO_LED_WLAN		14
#define TL_WVR300V4_GPIO_LED_SYSTEM	    12
#define TL_WVR300V4_KEYS_POLL_INTERVAL		20	/* msecs */
#define TL_WVR300V4_KEYS_DEBOUNCE_INTERVAL	(3 * TL_WVR300V4_KEYS_POLL_INTERVAL)
#define TL_WVR300V4_GPIO_LED_FIXED_WAN  15
#define TL_WVR300V4_GPIO_LED_FIXED_LAN  16
#define TL_WVR300V4_GPIO_74HC595_OE     3

static const char *tl_wvr300v4_part_probes[] = {
	"tp-link",
	NULL,
};

static struct flash_platform_data tl_wvr300v4_flash_data = {
	.part_probes	= tl_wvr300v4_part_probes,
};

#if 1
static struct gpio_led tl_wvr300v4_leds_gpio[] __initdata = {
	{
		.name		= "SYS_LED",
		.gpio		= TL_WVR300V4_GPIO_LED_SYSTEM,
		.active_low	= 1,
	}, {
		.name		= "WLAN2G",
		.gpio		= TL_WVR300V4_GPIO_LED_WLAN,
		.active_low	= 1,
	},
};

static struct gpio_keys_button tl_wvr300v4_gpio_keys[] __initdata = {
	{
		.desc		= "reset",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = TL_WVR300V4_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= TL_WVR300V4_GPIO_BTN_RESET,
		.active_low	= 1,
	}
};
#endif /* 0 */

void gpio_set_value(unsigned gpio, int value);
/********************************************************************************************/
/*                                74hc595d control                                          */
/********************************************************************************************/
#define tl_wvr300v4_74hc595d_val_bit(val, nr) (((val) & 0x7f & (1 << (nr))) >> (nr))
#define TL_WVR300V4_74HC595D_GPIO_DATA      0
#define TL_WVR300V4_74HC595D_GPIO_SHIFT     1
#define TL_WVR300V4_74HC595D_GPIO_OUTPUT    2
#define TL_WVR300V4_74HC595D_MAX_BITS       7
#define TL_WVR300V4_74HC595D_LED_WAN_OFF    3   /* 0b11 */
#define TL_WVR300V4_74HC595D_LED_GREEN      2   /* 0b10 */
#define TL_WVR300V4_74HC595D_LED_YELLOW     1   /* 0b01 */
#define TL_WVR300V4_74HC595D_LED_LAN_OFF    0   /* 0b00 */
#define TL_WVR300V4_74HC595D_BIT_EN         6
#define TL_WVR300V4_74HC595D_BIT_LED0       4   /* bit4 bit5 */
#define TL_WVR300V4_74HC595D_BIT_LED1       2   /* bit2 bit3 */
#define TL_WVR300V4_74HC595D_BIT_LED2       0   /* bit0 bit1 */


static u8 l_74hc595d_val = 0b0010101;
static u8 l_74hc595d_cur = 0b1000000;

static void __74hc595d_set_val(u8 val)
{
    int nr = 0;
    gpio_set_value(TL_WVR300V4_74HC595D_GPIO_OUTPUT, 0);

    for (nr = TL_WVR300V4_74HC595D_MAX_BITS - 1; nr >= 0; nr --)
    {
        gpio_set_value(TL_WVR300V4_74HC595D_GPIO_DATA, tl_wvr300v4_74hc595d_val_bit(val, nr));
        gpio_set_value(TL_WVR300V4_74HC595D_GPIO_SHIFT, 0);
        ndelay(20);
        gpio_set_value(TL_WVR300V4_74HC595D_GPIO_SHIFT, 1);
        ndelay(20);
    }

    gpio_set_value(TL_WVR300V4_74HC595D_GPIO_OUTPUT, 1);
}

void tl_wvr300v4_led_on(u8 led)
{
    u8 bit = (led & 3) << 1;

    l_74hc595d_cur = (l_74hc595d_cur & ~(3 << bit)) | (l_74hc595d_val & (3 << bit));
    __74hc595d_set_val(l_74hc595d_cur);
}

void tl_wvr300v4_led_off(u8 led)
{
    u8 bit = (led & 3) << 1;

    l_74hc595d_cur = l_74hc595d_cur & ~(3 << bit);
    __74hc595d_set_val(l_74hc595d_cur);
}

void tl_wvr300v4_led_revise(u8 led)
{
    u8 bit = (led & 3) << 1;

    if (l_74hc595d_cur & (3 << bit))
    {
        l_74hc595d_cur = l_74hc595d_cur & ~(3 << bit);
    }
    else
    {
        l_74hc595d_cur = (l_74hc595d_cur & ~(3 << bit)) | (l_74hc595d_val & (3 << bit));
    }
    __74hc595d_set_val(l_74hc595d_cur);
}

static u8 l_fixed_lan_led = 1;
static u8 l_fixed_wan_led = 1;
void gpio_led_on(u8 led)
{
    switch (led)
    {
    case 0:
        l_fixed_wan_led = 0;
        gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_WAN, l_fixed_wan_led);
        break;
    case 4:
        l_fixed_lan_led = 0;
        gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_LAN, l_fixed_lan_led);
        break;
    default:
        break;
    }

    return;
}

void gpio_led_off(u8 led)
{
    switch (led)
    {
    case 0:
        l_fixed_wan_led = 1;
        gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_WAN, l_fixed_wan_led);
        break;
    case 4:
        l_fixed_lan_led = 1;
        gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_LAN, l_fixed_lan_led);
        break;
    default:
        break;
    }

    return;
}

void gpio_led_revise(u8 led)
{
    switch (led)
    {
    case 0:
        l_fixed_wan_led = !l_fixed_wan_led;
        gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_WAN, l_fixed_wan_led);
        break;
    case 4:
        l_fixed_lan_led = !l_fixed_lan_led;
        gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_LAN, l_fixed_lan_led);
        break;
    default:
        break;
    }

    return;
}



static spinlock_t gpio_74hc595d_lock;

static int gpio_74hc595d_out_val(unsigned offset, int value)
{
    if (offset > TL_WVR300V4_74HC595D_MAX_BITS - 1)
    {
        return 0;
    }

    spin_lock(&gpio_74hc595d_lock);
    l_74hc595d_val &= ~(1 << offset);
    l_74hc595d_val |= (!!(value)) << offset;
    if (TL_WVR300V4_74HC595D_BIT_EN == offset)
    {
        l_74hc595d_cur &= ~(1 << offset);
        l_74hc595d_cur |= (!!(value)) << offset;
    }
    spin_unlock(&gpio_74hc595d_lock);

    return 0;
}

static int gpio_74hc595d_get_value(struct gpio_chip *chip, unsigned offset)
{
    if (offset > TL_WVR300V4_74HC595D_MAX_BITS - 1)
    {
        return 0;
    }
    
    return !!(l_74hc595d_val & (1 << offset));
}

static void gpio_74hc595d_set_value(struct gpio_chip *chip, unsigned offset, int value)
{
    gpio_74hc595d_out_val(offset, value);
}

static int gpio_74hc595d_direction_input(struct gpio_chip *chip, unsigned offset)
{
    return 0;
}

static int gpio_74hc595d_direction_output(struct gpio_chip *chip, unsigned offset, int value)
{
    gpio_74hc595d_out_val(offset, value);
    return 0;
}

static struct gpio_chip gpio_74hc595d = {
    .label            = "74hc595d",
    .get              = gpio_74hc595d_get_value,
    .set              = gpio_74hc595d_set_value,
    .to_irq           = NULL,
    .direction_input  = gpio_74hc595d_direction_input,
    .direction_output = gpio_74hc595d_direction_output,
    .base             = -1,
    .ngpio            = TL_WVR300V4_74HC595D_MAX_BITS,
};

int gpio_74hc595d_init(void)
{
    spin_lock_init(&gpio_74hc595d_lock);

    gpio_request(TL_WVR300V4_74HC595D_GPIO_DATA,   "74HC595D_DATA");
    gpio_request(TL_WVR300V4_74HC595D_GPIO_SHIFT,  "74HC595D_SHIFT");
    gpio_request(TL_WVR300V4_74HC595D_GPIO_OUTPUT, "74HC595D_OUTPUT");

    if (gpiochip_add(&gpio_74hc595d))
    {
        panic("cannot add 74HC595D gpio chip");
    }

    return 0;
}

/********************************************************************************************/
/*                                      end                                                 */
/********************************************************************************************/


static void __init tl_wvr300v4_setup(void)
{
	u8 *art = (u8 *) KSEG1ADDR(0x1fff0000);

	ath79_register_m25p80(&tl_wvr300v4_flash_data);

    ath79_gpio_function_enable(1<<1);
	ath79_gpio_direction_select(TL_WVR300V4_GPIO_74HC595_OE, true);
	ath79_gpio_direction_select(TL_WVR300V4_GPIO_LED_SYSTEM, true);
	ath79_gpio_direction_select(TL_WVR300V4_GPIO_LED_WLAN, true);
	ath79_gpio_direction_select(TL_WVR300V4_GPIO_LED_FIXED_WAN, true);
	ath79_gpio_direction_select(TL_WVR300V4_GPIO_LED_FIXED_LAN, true);
	ath79_gpio_direction_select(TL_WVR300V4_GPIO_BTN_RESET, false);
    gpio_set_value(TL_WVR300V4_GPIO_LED_SYSTEM, 1);
    gpio_set_value(TL_WVR300V4_GPIO_LED_WLAN, 1);
    gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_LAN, 1);
    gpio_set_value(TL_WVR300V4_GPIO_LED_FIXED_WAN, 1);
    /*
    ath79_gpio_output_select(TL_WVR300V4_GPIO_LED_FIXED_WAN, QCA953X_GPIO_OUT_MUX_LED_LINK1);
    ath79_gpio_output_select(TL_WVR300V4_GPIO_LED_FIXED_LAN, QCA953X_GPIO_OUT_MUX_LED_LINK5);
    */
	ath79_register_leds_gpio(-1, ARRAY_SIZE(tl_wvr300v4_leds_gpio),
				 tl_wvr300v4_leds_gpio);
	ath79_register_gpio_keys_polled(-1, TL_WVR300V4_KEYS_POLL_INTERVAL,
					 ARRAY_SIZE(tl_wvr300v4_gpio_keys),
					 tl_wvr300v4_gpio_keys);
    tl_wvr300v4_led_off(0);
    tl_wvr300v4_led_off(1);
    tl_wvr300v4_led_off(2);
    gpio_set_value(TL_WVR300V4_GPIO_74HC595_OE, 0);
    gpio_74hc595d_init();

	/*ath79_register_usb();*/
	/*ath79_register_pci();*/
	ath79_register_wmac(art + AP147_WMAC_CALDATA_OFFSET, NULL);

	ath79_register_mdio(0, 0x0);
	ath79_register_mdio(1, 0x0);

	ath79_init_mac(ath79_eth0_data.mac_addr, art + AP147_MAC0_OFFSET, 0);
	ath79_init_mac(ath79_eth1_data.mac_addr, art + AP147_MAC1_OFFSET, 0);

	/* LAN ports */
	ath79_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_GMII;
	ath79_eth1_data.speed = SPEED_1000;
	ath79_eth1_data.duplex = DUPLEX_FULL;
#if 0
	ath79_switch_data.phy_poll_mask |= BIT(4);
	ath79_switch_data.phy4_mii_en = 1;
#else
	ath79_switch_data.phy_poll_mask |= BIT(5);
#endif /* 0 */
	ath79_register_eth(1);

	/* WAN port */
#if 0
	ath79_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_MII;
	ath79_eth0_data.speed = SPEED_100;
	ath79_eth0_data.duplex = DUPLEX_FULL;
	ath79_eth0_data.phy_mask = BIT(4);
	ath79_register_eth(0);
#endif /* 0 */

}

MIPS_MACHINE(ATH79_MACH_TL_WVR300V4, "TL-WVR300V4", "TP-LINK TL-WVR300 V4",
	     tl_wvr300v4_setup);
