/*
 * Copyright (C) 2011 Gabor Juhos <juhosg@openwrt.org>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/magic.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>

#define TPLINK_NUM_PARTS	5
#define TPLINK_HEADER_V1	0x01000000
#define MD5SUM_LEN		16

#define TPLINK_ART_LEN		0x10000
#define TPLINK_KERNEL_OFFS	0x20000

struct tplink_fw_header {
	uint32_t	version;	/* header version */
	char		vendor_name[24];
	char		fw_version[36];
	uint32_t	hw_id;		/* hardware id */
	uint32_t	hw_rev;		/* hardware revision */
	uint32_t	unk1;
	uint8_t		md5sum1[MD5SUM_LEN];
	uint32_t	unk2;
	uint8_t		md5sum2[MD5SUM_LEN];
	uint32_t	unk3;
	uint32_t	kernel_la;	/* kernel load address */
	uint32_t	kernel_ep;	/* kernel entry point */
	uint32_t	fw_length;	/* total length of the firmware */
	uint32_t	kernel_ofs;	/* kernel data offset */
	uint32_t	kernel_len;	/* kernel data length */
	uint32_t	rootfs_ofs;	/* rootfs data offset */
	uint32_t	rootfs_len;	/* rootfs data length */
	uint32_t	boot_ofs;	/* bootloader data offset */
	uint32_t	boot_len;	/* bootloader data length */
	uint8_t		pad[360];
} __attribute__ ((packed));

static struct tplink_fw_header *
tplink_read_header(struct mtd_info *mtd, size_t offset)
{
	struct tplink_fw_header *header;
	size_t header_len;
	size_t retlen;
	int ret;
	u32 t;

	header = vmalloc(sizeof(*header));
	if (!header)
		goto err;

	header_len = sizeof(struct tplink_fw_header);
	ret = mtd_read(mtd, offset, header_len, &retlen,
		       (unsigned char *) header);
	if (ret)
		goto err_free_header;

	if (retlen != header_len)
		goto err_free_header;

	/* sanity checks */
	t = be32_to_cpu(header->version);
	if (t != TPLINK_HEADER_V1)
		goto err_free_header;

	t = be32_to_cpu(header->kernel_ofs);
	if (t != header_len)
		goto err_free_header;

	return header;

err_free_header:
	vfree(header);
err:
	return NULL;
}

static int tplink_check_rootfs_magic(struct mtd_info *mtd, size_t offset)
{
	u32 magic;
	size_t retlen;
	int ret;

	ret = mtd_read(mtd, offset, sizeof(magic), &retlen,
		       (unsigned char *) &magic);
	if (ret)
		return ret;

	if (retlen != sizeof(magic))
		return -EIO;

	if (le32_to_cpu(magic) != SQUASHFS_MAGIC &&
	    magic != 0x19852003)
		return -EINVAL;

	return 0;
}
#if defined(CONFIG_ATH79_MACH_TL_WVR450G)
static int tplink_parse_partitions(struct mtd_info *master,
				   struct mtd_partition **pparts,
				   struct mtd_part_parser_data *data)
{
	struct mtd_partition *parts;
	int nr_parts;
	int ret;

	printk("@@-----debug wvr450g--tplink flash parse--------\n");
	nr_parts = TPLINK_NUM_PARTS;
	parts = kzalloc(nr_parts * sizeof(struct mtd_partition), GFP_KERNEL);
	if (!parts) {
		ret = -ENOMEM;
		*pparts = NULL;
		return ret;
	}

	parts[0].name = "u-boot";
	parts[0].offset = 0;
	parts[0].size = 0x20000;

	parts[1].name = "kernel";
	parts[1].offset = 0x20000;
	parts[1].size = 0x300000;

	parts[2].name = "rootfs";
	parts[2].offset = 0x320000;
	parts[2].size = 0x4e0000;

	parts[3].name = "firmware";
	parts[3].offset = 0x20000;
	parts[3].size = 0x6a0000;


	*pparts = parts;
	return nr_parts;
}

#elif defined(CONFIG_ATH79_MACH_TL_WVR1300G) || defined(CONFIG_ATH79_MACH_TL_R478GV2) || defined(CONFIG_ATH79_MACH_TL_WVR300V4) || defined(CONFIG_ATH79_MACH_TL_WVR450V1) || defined(CONFIG_ATH79_MACH_TL_R4299GV2)\
|| defined(CONFIG_ATH79_MACH_TL_R473P_ACV1)|| defined(CONFIG_ATH79_MACH_TL_R479P_ACV1)
static int tplink_parse_partitions(struct mtd_info *master,
				   struct mtd_partition **pparts,
				   struct mtd_part_parser_data *data)
{
	struct mtd_partition *parts;
	int nr_parts;
	int ret;

	printk(KERN_ERR"@@-----debug wvr1300g--tplink flash parse--------\n");
	nr_parts = 12;
	parts = kzalloc(nr_parts * sizeof(struct mtd_partition), GFP_KERNEL);
	if (!parts) {
		ret = -ENOMEM;
		*pparts = NULL;
		return ret;
	}

#ifdef CONFIG_DKMGT_DEFAULT
	parts[0].name   = "bootloader";
	parts[0].offset = 0x00000000;
	parts[0].size   = 0x00040000;

    parts[1].name   = "kernel";
	parts[1].offset = 0x00040000;
	parts[1].size   = 0x00150000;

    parts[2].name   = "rootfs";
	parts[2].offset = 0x00190000;
	parts[2].size   = 0x00ba0000;

    parts[3].name   = "device-info";
	parts[3].offset = 0x00d30000;
	parts[3].size   = 0x00010000;

    parts[4].name   = "firmware-info";
	parts[4].offset = 0x00d40000;
	parts[4].size   = 0x00010000;

    parts[5].name   = "tddp";
	parts[5].offset = 0x00d50000;
	parts[5].size   = 0x00010000;

    parts[6].name   = "log";
	parts[6].offset = 0x00d60000;
	parts[6].size   = 0x00020000;

    parts[7].name   = "rootfs_data";
	parts[7].offset = 0x00df0000;
	parts[7].size   = 0x00200000;

    parts[8].name   = "art";
	parts[8].offset = 0x00ff0000;
	parts[8].size   = 0x00010000;

	parts[9].name  = "firmware";
	parts[9].offset= 0x00000000;
	parts[9].size  = 0x01000000; /* the whole 16MB flash. */  
#endif

#ifdef CONFIG_DKMGT_CLOUD
	parts[0].name   = "bootloader";
	parts[0].offset = 0x00000000;
	parts[0].size   = 0x00040000;

    parts[1].name   = "partition-table";
	parts[1].offset = 0x00040000;
	parts[1].size   = 0x00010000;

    parts[2].name   = "kernel";
	parts[2].offset = 0x00050000;
	parts[2].size   = 0x00150000;

    parts[3].name   = "device-info";
	parts[3].offset = 0x001a0000;
	parts[3].size   = 0x00010000;

    parts[4].name   = "firmware-info";
	parts[4].offset = 0x001b0000;
	parts[4].size   = 0x00010000;

    parts[5].name   = "tddp";
	parts[5].offset = 0x001c0000;
	parts[5].size   = 0x00010000;

    parts[6].name   = "rootfs";
	parts[6].offset = 0x001d0000;
	parts[6].size   = 0x00d60000;

    parts[7].name   = "user-config";
	parts[7].offset = 0x00f30000;
	parts[7].size   = 0x00090000;

    parts[8].name   = "support-list";
	parts[8].offset = 0x00fc0000;
	parts[8].size   = 0x00010000; 
	
    parts[9].name   = "extra-para";
	parts[9].offset = 0x00fd0000;
	parts[9].size   = 0x00010000; 
	
	parts[10].name   = "ART";
	parts[10].offset = 0x00fe0000;
	parts[10].size   = 0x00020000; 
	
	parts[11].name  = "firmware";
	parts[11].offset= 0x00000000;
	parts[11].size  = 0x01000000; /* the whole 16MB flash. */  
#endif
	
    *pparts = parts;
	return nr_parts;
}
#else
static int tplink_parse_partitions(struct mtd_info *master,
				   struct mtd_partition **pparts,
				   struct mtd_part_parser_data *data)
{
	struct mtd_partition *parts;
	struct tplink_fw_header *header;
	int nr_parts;
	size_t offset;
	size_t art_offset;
	size_t rootfs_offset;
	size_t squashfs_offset;
	int ret;

	nr_parts = TPLINK_NUM_PARTS;
	parts = kzalloc(nr_parts * sizeof(struct mtd_partition), GFP_KERNEL);
	if (!parts) {
		ret = -ENOMEM;
		goto err;
	}

	offset = TPLINK_KERNEL_OFFS;

	header = tplink_read_header(master, offset);
	if (!header) {
		pr_notice("%s: no TP-Link header found\n", master->name);
		ret = -ENODEV;
		goto err_free_parts;
	}

	squashfs_offset = offset + sizeof(struct tplink_fw_header) +
			  be32_to_cpu(header->kernel_len);

	ret = tplink_check_rootfs_magic(master, squashfs_offset);
	if (ret == 0)
		rootfs_offset = squashfs_offset;
	else
		rootfs_offset = offset + be32_to_cpu(header->rootfs_ofs);

	art_offset = master->size - TPLINK_ART_LEN;

	parts[0].name = "u-boot";
	parts[0].offset = 0;
	parts[0].size = offset;
	parts[0].mask_flags = MTD_WRITEABLE;

	parts[1].name = "kernel";
	parts[1].offset = offset;
	parts[1].size = rootfs_offset - offset;

	parts[2].name = "rootfs";
	parts[2].offset = rootfs_offset;
	parts[2].size = art_offset - rootfs_offset;

	parts[3].name = "art";
	parts[3].offset = art_offset;
	parts[3].size = TPLINK_ART_LEN;
	parts[3].mask_flags = MTD_WRITEABLE;

	parts[4].name = "firmware";
	parts[4].offset = offset;
	parts[4].size = art_offset - offset;

	vfree(header);

	*pparts = parts;
	return nr_parts;

err_free_parts:
	kfree(parts);
err:
	*pparts = NULL;
	return ret;
}
#endif

static struct mtd_part_parser tplink_parser = {
	.owner		= THIS_MODULE,
	.parse_fn	= tplink_parse_partitions,
	.name		= "tp-link",
};

static int __init tplink_parser_init(void)
{
	return register_mtd_parser(&tplink_parser);
}

module_init(tplink_parser_init);

MODULE_LICENSE("GPL v2");
MODULE_AUTHOR("Gabor Juhos <juhosg@openwrt.org>");
