#!/bin/sh
# This script is executed as part of the hotplug event with
# HOTPLUG_TYPE=iface, triggered by various scripts when an interface
# is configured (ACTION=ifup) or deconfigured (ACTION=ifdown).  The
# interface is available as INTERFACE, the real device as DEVICE.

[ ${INTERFACE:$((${#INTERFACE}-2))} == "v6" ] && exit 0
. /lib/zone/zone_api.sh
. /lib/functions.sh
. /lib/nat/nat_core.sh

if [[ -z "$INTERFACE" ]]; then
	[ -z "$DEVICE" ] && exit 0
	INTERFACE=`zone -F zone_get_iface_bydev -P $DEVICE`
fi

[ -z "$INTERFACE" ] && exit 0 
[ "$DEVICE" == "lo" ] && exit 0

mkdir -p /tmp/.nat
local HOTP_LOG="/tmp/.nat/hotplug.log"
if [[ "`ls -l $HOTP_LOG | awk '{print $5}'`" -gt 50000 ]]; then
	echo -n "" > $HOTP_LOG
fi

HP_LOG()
{
	echo -e "`date +%Y%m%d-%H:%M:%S` $@" >> $HOTP_LOG
}
HP_LOG "$INTERFACE $ACTION"

{
	[ "$INTERFACE" == "lan" ] && {
		nat_napt_iface_list_hotplug $ACTION $DEVICE
	}
}

{
	flock -x 50
	if [[ ! -f "$NAT_READY" ]]; then
		HP_LOG "NAT is not ready, ingore this event"
		exit 0
	fi
	flock -u 50
} 50<>/tmp/nat.lock

local ruleNum=0
local ruleNumVs=0
local ruleNumPt=0
local ruleNumDmz=0
local ruleNumOnenat=0
local needSave=0

# delete one nat rule when wan iface is not static
modifyOnenatRule()
{
	local cfg="$1"
	local iface="$2"
	local zone
	local interface

	config_get zone "$cfg" interface
	nat_get_effect_ifaces "$zone" interface
	if [[ "$interface" == "$iface" ]]; then
		uci delete nat.$cfg
		needSave=1
	fi
}

# modify multinat sys rule when lan ip changed
modifyNatSysRule()
{
	local cfg="$1"
	local subnet="$2"
	local mask="$3"
	local sysRuleFlag
	local oldIpaddr
	local oldMask

	config_get oldIpaddr "$cfg" ipaddr
	config_get oldMask "$cfg" mask
	config_get sysRuleFlag "$cfg" sysrule

	if [[ "$oldIpaddr" != "$subnet" || "$oldMask" != "$mask" ]]; then	
		if [[ -n "$sysRuleFlag" && "$sysRuleFlag" = "1" ]]; then
			uci set nat.$1.mask=${mask}
			uci set nat.$1.ipaddr=${subnet}
			needSave=1
		fi
	fi
}

lanIfHotPlugEvent()
{
	local lanIf="lan"
	if [[ -n "${INTERFACE}" && "${INTERFACE}" == "${lanIf}" ]]; then
		HP_LOG "LAN interface $ACTION, modify sys napt rules, interface: ${INTERFACE}"
		local ipaddr
		local subnet
		local netmask
		local prefix

		local zone=`zone -F zone_get_zone_byif -P "$INTERFACE"`
		if [ -n $zone ]; then
			nat_config_load
			config_foreach nat_get_rule_num rule_vs "rule_vs" $zone
			config_foreach nat_get_rule_num rule_dmz "rule_dmz" $zone
			config_foreach nat_get_rule_num rule_onenat "rule_onenat" $zone
			config_foreach nat_get_rule_num rule_pt "rule_pt" $zone
		fi

		ipaddr=$(nat_napt_lan_ipaddr)
		netmask=$(nat_napt_lan_netmask)
		if [[ -z "$ipaddr" || -z "$netmask" ]]; then
			HP_LOG "lan subnet is null error"
			return
		else
			HP_LOG "ipaddr: $ipaddr, netmask: $netmask"
			prefix=$(ipcalc -p $ipaddr $netmask)
			prefix=${prefix#*=}
			subnet=$(ipcalc -n $ipaddr $netmask)
			subnet=${subnet#*=}
			HP_LOG "prefix: $prefix, subnet: $subnet"
		fi

		nat_config_load
		config_foreach modifyNatSysRule rule_napt $subnet $prefix

		if [[ "$needSave" -eq 1 ]]; then
			uci commit nat
			/lib/cfg_save/config.sh save
		fi

		HP_LOG "restart nat"
		if [[ "$ruleNum" -gt 0 ]]; then
			/etc/init.d/nat restart
		else
			/etc/init.d/nat restart napt
		fi
	fi
}

wanIfHotPlugEvent()
{
	local lanIf="lan"

	if [[ -n "${INTERFACE}" && "${INTERFACE}" != "${lanIf}" ]]; then
		HP_LOG "WAN interface $ACTION, modify one nat rules if needed, interface: ${INTERFACE}"

		local proto=`uci get network.${INTERFACE}.proto`
		if [[ "$proto" != "static" ]]; then
			nat_config_load
			config_foreach modifyOnenatRule rule_onenat ${INTERFACE}
		fi

		if [[ "$needSave" -eq 1 ]]; then
			uci commit nat
			/lib/cfg_save/config.sh save
			/etc/init.d/nat restart onenat
		fi

		nat_hotplug_check_rule "$INTERFACE" "$ACTION" 0
	fi
}

wan_add_mark()
{
	HP_LOG "!!!! add iptables ${DEVICE} ${wan_mark}"
	iptables -w -t mangle -N Add_mark_${INTERFACE}
	iptables -w -t mangle -I POSTROUTING 1 -j Add_mark_${INTERFACE}
	iptables -w -t mangle -I Add_mark_${INTERFACE} 1 -o ${DEVICE} -j MARK --or-mark ${wan_mark}
}

wan_del_mark()
{
	HP_LOG "!!!! del iptables ${DEVICE} ${wan_mark}"
	iptables -w -t mangle -F Add_mark_${INTERFACE}
	iptables -w -t mangle -D POSTROUTING -j Add_mark_${INTERFACE}
}

local need_wan_mark=`uci -q -c /etc/profile.d get profile.@global[0].need_wan_mark`
local wan_mark
[ "x$need_wan_mark" = "x1" ] && {
	local port_id=`echo $INTERFACE | tr -cd "[0-9]"`
	[ "x$port_id" = 'x1' ] && wan_mark="0x2000"
	[ "x$port_id" = 'x2' ] && wan_mark="0xc000"
	[ "x$port_id" = 'x3' ] && wan_mark="0x4000"
	[ "x$port_id" = 'x4' ] && wan_mark="0x6000"
	[ "x$port_id" = 'x5' ] && wan_mark="0x8000"
	[ "x$port_id" = 'x6' ] && wan_mark="0xa000"
}

case "$ACTION" in
	ifup)
		lanIfHotPlugEvent
		wanIfHotPlugEvent
		[ -n "${wan_mark}" ] && [ "${INTERFACE:0:3}" == "wan" ] && wan_add_mark
	;;
	ifdown)
		lanIfHotPlugEvent
		wanIfHotPlugEvent
		[ -n "${wan_mark}" ] && [ "${INTERFACE:0:3}" == "wan" ] && wan_del_mark
	;;
esac
