local ctl = require "luci.model.controller"
local uci = require "luci.model.uci"
local sys = require "luci.sys"
local form = require "luci.tools.form"
local dbg = require "luci.tools.debug"
local err = require "luci.tools.error"
local json = require "luci.json"
local dt_m = require "luci.tools.datatypes"
local utl = require "luci.util"
local ifs = require "luci.model.interfaces".ifs()
local ifaces = require "luci.model.interfaces"
local qos = require "luci.model.qos"
local userconfig   = require "luci.model.userconfig"
local ipsec_utility= require "luci.controller.admin.ipsec"
local ip = require "luci.ip"
local pppox = require "luci.controller.admin.pppox"
local error_print = require "luci.tools.error_print"
local dtype = require "luci.tools.datatypes"
local ippoolref = require "luci.model.reference"

local reloadl2tp = "/lib/l2tp/l2tp-reload.sh"
local restartqos = "/etc/init.d/qos-tplink"

local user_configname = "user-secrets"

local uci_r = uci.cursor()
Form = form.Form(uci_r, {"id"})
ippoolref = ippoolref.REF_INST()

local server_config = "l2tp-server"
local client_config = "l2tp-client"

local global_config = "l2tp-global"

local MAX_L2TP_INTERVAL	= 1000
local MAX_INTERVAL = 120
local PSK_MAXLEN = 128
local IF_MAXLEN = 64
local TUNNELNAME_MAXLEN = 16

local USERNAME_MAXLEN = 64
local PASSWORD_MAXLEN = 128

local server_list_max = 200
local client_list_max = 32
local max_session = 100

function get_max_sessions()
    local ret

    if uci_r:get_profile("l2tp", "tunnel_max") then
        ret = uci_r:get_profile("l2tp", "tunnel_max") > max_session and max_session or uci_r:get_profile("l2tp", "tunnel_max")
    else
        ret = max_session
    end
    return ret
end

local function find_section(cfg, stype, index)
    local msection = { }
    local count = 0

    index =  type(index) == "table" and index or { tonumber(index) }

    uci_r:foreach(cfg, stype,
        function(section)
            for i, v in ipairs(index) do
                if count == tonumber(v) then
                    msection[count + 1] = section
                    -- del options generated by uci (they are not mine)
                    for k, v in pairs(msection[count + 1]) do
                        if k:sub(1, 1) == "." then
                            msection[k] = nil
                        end
                    end
                end
            end
            count = count + 1
        end
    )

    return msection

end

function add_l2tp_server(http_form)
    local new = json.decode(http_form)
    local error_code = nil
    local ret_ipsec = nil
    local c_flag = 0;

    if new.cid ~= nil then
        c_flag = 1
    end

    cmd = reloadl2tp .. " add lns " .. dtype.safeSpecialString(new.id) .. " " .. dtype.safeSpecialString(new.enable)
    cmd = cmd .."; ipsec reload"
    sys.fork_call(cmd)

    if new.authtype == "ldap" then
        local add_form = {}
        local ldap_name = "@ldap-l2tp".."-"..new.id
        local ldap_password = "@secret"
        local ldap_type = "l2tp"
        local new2 ={}
        --if new.netmode == "lan2lan" then
            if c_flag == 0 then --standalone
                new2 =
                {
                    ["username"] = ldap_name,
                    ["password"] = ldap_password,
                    ["type"] = ldap_type,
                    ["localip"] = "172.31."..tostring(math.floor(new.id_ip/255)).."."..tostring(new.id_ip % 255),
                    ["ippool"] = new.ippool,
                    ["ms_dns1"] = new.ms_dns1,
                    ["ms_dns2"] = new.ms_dns2,
                    ["netmode"] = new.netmode,
                    ["remotesubnet"] = new.remotesubnet,
                    ["maxsessions"] = new.maxsessions,
                    ["networktype"] = new.networktype,
                    ["local_network"] = new.local_network,
                }
            else --controller
                new2 =
                {
                    ["username"] = ldap_name,
                    ["password"] = ldap_password,
                    ["type"] = ldap_type,
                    ["localip"] = "172.31."..tostring(math.floor(new.id_ip/255)).."."..tostring(new.id_ip % 255),
                    ["ippool"] = new.ippool,
                    ["ms_dns1"] = new.ms_dns1,
                    ["ms_dns2"] = new.ms_dns2,
                    ["netmode"] = "client2lan",
                    ["local_network"] =new.local_network,
                    ["maxsessions"] = tostring(get_max_sessions()),
                    ["networktype"] = "",
                }
            end
        add_form.data= json.encode(
            {
                ["method"] = "add",
                ["params"] =
                {
                    ["index"] = "1",
                    ["old"] = "add",
                    ["new"] = new2,
                    ["key"] = "add",
                }
            }
        )
        add_form.form = "usermanager"
        local ret = pppox.dispatch(add_form)
        if (not ret) or (ret.error_code ~= "0") then
            dbg.print_tty(error_print.check_error(ret))
            return "FAILED"
        end

        --add ldap ref
        local ref = 0
        ref = tonumber(uci_r:get("ldap",new.ldap_profile,"ref"))
        ref = ref + 1
        uci_r:set("ldap", new.ldap_profile, "ref", tostring(ref))
        if not uci_r:commit("ldap") then
            return false, err.ERR_COM_UCI_COMMIT
        end
    end
    local controller_mode = io.open("/tmp/pppox.controller.mode","r")
    if controller_mode then
        cmd = ". /lib/pppox/pppox-functions.sh && pppox_load_user" .. " l2tp " .. new.id
        sys.fork_call(cmd)
        cmd = ". /lib/pppox/pppox-remote-management.sh; refresh_vpn_remote_management;"
        sys.fork_call(cmd)
        controller_mode:close()
    end

    userconfig.cfg_modify()
    return ret
end

function delete_l2tp_server(http_form)
    local input = json.decode(http_form)

    local index = input.index
    local cmd = input.cmd
    local refifs = input.refifs

    local error_code = nil
    local ret_ipsec = nil

    index=utl.split(tostring(input.index),',')
    index = (type(index)=="table") and index or {index}
    refifs = (type(refifs)=="table") and refifs or {refifs}
    local servername = utl.split(input.name,',')
    servername = (type(servername)=="table") and servername or {servername}

    sys.fork_call(cmd)

    local infaces = ""
    if next(refifs) ~= nil then
        infaces = table.concat(refifs, " ")
    end

    sys.fork_call(". /lib/pppox/api.sh && deal_with_qos_ref \"%s\"" % {dtype.safeSpecialString(infaces)})

    uci_r:foreach(user_configname,"user",
                    function(section)
                        local tmp = uci_r:get_all(user_configname,section[".name"])
                        if string.find(tmp["username"],"@ldap",1)  then
                            for _,v in pairs(servername) do
                                if tmp["username"] == ("@ldap-l2tp-"..v) then
                                    ippoolref:update(0,tmp["ippool"])
                                    uci_r:delete(user_configname,section[".name"])
                                end
                            end
                        end
                    end
    )
    uci_r:commit(user_configname)
    userconfig.cfg_modify()
    return ret

end

function delete_l2tp_client(http_form)
    local input = json.decode(http_form)

    local index = input.index
    local refifs = input.refifs
    local tnifs = input.tnifs
    local cmd = input.cmd

    local error_code = nil
    local ret_ipsec = nil

    refifs = (type(refifs)=="table") and refifs or {refifs}
    tnifs = (type(tnifs)=="table") and tnifs or {tnifs}

    index=utl.split(tostring(input.index),',')
    index = (type(index)=="table") and index or {index}

    local mold = find_section(client_config, "lac", index)

    local ret = {}

    local infaces = ""
    local qoss = ""
    if next(refifs) ~= nil then
        infaces = table.concat(refifs, " ")
    end

    if next(tnifs) ~= nil then
        qoss = table.concat(tnifs, " ")
    end

    local sep = ";"
    if cmd:sub(-1,-1) == ";" then
        sep = ""
    end

    cmd = cmd .. sep .. "/etc/init.d/zone restart"
    cmd = cmd .. ";" .. ". /lib/pppox/api.sh && deal_with_qos_ref \"%s\" \"%s\""
    cmd = cmd .. ";" .. restartqos .. " start"
    sys.fork_call(cmd % {infaces, qoss})

    userconfig.cfg_modify();
    return true

end

function add_l2tp_client(http_form)
    local new = json.decode(http_form)

    -- add interface info into qos model
    if not qos.add_interface(new.tunnelname, tonumber(new.uplink), tonumber(new.downlink)) then
        return false, err.ERR_COM_TABLE_ITEM_UCI_ADD
    end

    cmd = "/etc/init.d/zone restart"
    cmd = cmd .. ";" .. reloadl2tp .. " add lac " .. dtype.safeSpecialString(new.id) .. " " .. dtype.safeSpecialString(new.enable)
    cmd = cmd .. ";" .. restartqos .. " start"
    sys.fork_call(cmd)

    userconfig.cfg_modify();
    return ret

end



local type = arg[1]
local action = arg[2]
local http = arg[3]

if type == "client" then
    if action == "add" then
        add_l2tp_client(http)
    elseif action == "del" then
        delete_l2tp_client(http)
    end
elseif type == "server" then
    if action == "add" then
        add_l2tp_server(http)
    elseif action == "del" then
        delete_l2tp_server(http)
    end
end